@@ -2,6 +2,7 @@
 #
 # SPDX-License-Identifier: MPL-2.0
 """This module contains the utilities."""
+import asyncio
 from collections import defaultdict
 from contextlib import asynccontextmanager
 from functools import wraps
@@ -39,7 +40,7 @@ def handle_exclusively(key: Callable[..., Hashable]) -> Callable:
     Examples:
         Simple usage::
 
-            @handle_exclusively(key=lambda **kwargs: (kwargs["x"], kwargs["y"]))
+            @handle_exclusively(key=lambda x, y, z: x, y)
             async def f(x, y, z):
                 pass
 
@@ -59,18 +60,52 @@ def handle_exclusively(key: Callable[..., Hashable]) -> Callable:
     async def named_lock(key: Hashable) -> AsyncGenerator[None, None]:
         lock = locks[key]
         async with lock:
-            yield
-            # Garbage collect lock if no others are waiting to acquire.
-            # This MUST be done before we release the lock.
-            if not lock.statistics().tasks_waiting:
-                del locks[key]
+            try:
+                yield
+            finally:
+                # Garbage collect lock if no others are waiting to acquire.
+                # This MUST be done before we release the lock.
+                if not lock.statistics().tasks_waiting:
+                    del locks[key]
 
     def wrapper(coro: Callable[..., Awaitable[T]]) -> Callable[..., Awaitable[T]]:
         @wraps(coro)
         async def wrapped(*args: Any, **kwargs: Any) -> T:
-            async with named_lock(key=key(**kwargs)):
+            async with named_lock(key=key(*args, **kwargs)):
                 return await coro(*args, **kwargs)
 
         return wrapped
 
     return wrapper
+
+
+@asynccontextmanager
+async def sleep_on_error(delay: int = 30) -> AsyncGenerator:
+    """Async context manager that delays returning on errors.
+
+    This is used to prevent race-conditions on writes to MO/LoRa, when the upload times
+    out initially but is completed by the backend afterwards. The sleep ensures that
+    the AMQP message is not retried immediately, causing the handler to act on
+    information which could become stale by the queued write. This happens because the
+    backend does not implement fairness of requests, such that read operations can
+    return soon-to-be stale data while a write operation is queued on another thread.
+
+    Specifically, duplicate objects would be created when a write operation failed to
+    complete within the timeout (but would be completed later), and the handler, during
+    retry, read an outdated list of existing objects, and thus dispatched another
+    (duplicate) write operation.
+
+    See: https://redmine.magenta-aps.dk/issues/51949#note-23.
+    """
+    try:
+        yield
+    except Exception:  # pylint: disable=broad-except
+        await asyncio.sleep(delay)
+
+
+class RejectMessage(Exception):
+    """Raise to reject a message, turning it into a dead letter."""
+
+
+class RequeueMessage(Exception):
+    """Raise to requeue a message for later redelivery."""
